@@ -41,13 +41,14 @@ static void dump_folder_list(LIBMTP_folder_t *folderlist, int level)
 
 int main (int argc, char **argv)
 {
-  LIBMTP_mtpdevice_t *device, *iter;
+  LIBMTP_raw_device_t *rawdevices;
+  int numrawdevices;
+  int i;
 
   LIBMTP_Init();
   printf("Attempting to connect device(s)\n");
 
-  switch(LIBMTP_Get_Connected_Devices(&device))
-  {
+  switch (LIBMTP_Detect_Raw_Devices(&rawdevices, &numrawdevices)) {
   case LIBMTP_ERROR_NO_DEVICE_ATTACHED:
     printf("mtp-folders: no devices found\n");
     return 0;
@@ -63,7 +64,7 @@ int main (int argc, char **argv)
   default:
     fprintf(stderr, "mtp-folders: Unknown error, please report "
                     "this to the libmtp developers\n");
-  return 1;
+    return 1;
 
   /* Successfully connected at least one device, so continue */
   case LIBMTP_ERROR_NONE:
@@ -71,14 +72,20 @@ int main (int argc, char **argv)
   }
 
   /* iterate through connected MTP devices */
-  for(iter = device; iter != NULL; iter = iter->next)
-  {
+  for (i = 0; i < numrawdevices; i++) {
+    LIBMTP_mtpdevice_t *device;
     LIBMTP_devicestorage_t *storage;
     char *friendlyname;
     int ret;
 
+    device = LIBMTP_Open_Raw_Device(&rawdevices[i]);
+    if (device == NULL) {
+      fprintf(stderr, "Unable to open raw device %d\n", i);
+      continue;
+    }
+
     /* Echo the friendly name so we know which device we are working with */
-    friendlyname = LIBMTP_Get_Friendlyname(iter);
+    friendlyname = LIBMTP_Get_Friendlyname(device);
     if (friendlyname == NULL) {
       printf("Friendly name: (NULL)\n");
     } else {
@@ -86,15 +93,15 @@ int main (int argc, char **argv)
       free(friendlyname);
     }
 
-    LIBMTP_Dump_Errorstack(iter);
-    LIBMTP_Clear_Errorstack(iter);
+    LIBMTP_Dump_Errorstack(device);
+    LIBMTP_Clear_Errorstack(device);
 
     /* Get all storages for this device */
     ret = LIBMTP_Get_Storage(device, LIBMTP_STORAGE_SORTBY_NOTSORTED);
     if (ret != 0) {
       perror("LIBMTP_Get_Storage()\n");
-      LIBMTP_Dump_Errorstack(iter);
-      LIBMTP_Clear_Errorstack(iter);
+      LIBMTP_Dump_Errorstack(device);
+      LIBMTP_Clear_Errorstack(device);
       continue;
     }
 
@@ -103,20 +110,21 @@ int main (int argc, char **argv)
       LIBMTP_folder_t *folders;
 
       printf("Storage: %s\n", storage->StorageDescription);
-      folders = LIBMTP_Get_Folder_List_For_Storage(iter, storage->id);
+      folders = LIBMTP_Get_Folder_List_For_Storage(device, storage->id);
 
       if (folders == NULL) {
 	fprintf(stdout, "No folders found\n");
-	LIBMTP_Dump_Errorstack(iter);
-	LIBMTP_Clear_Errorstack(iter);
+	LIBMTP_Dump_Errorstack(device);
+	LIBMTP_Clear_Errorstack(device);
       } else {
 	dump_folder_list(folders,0);
       }
       LIBMTP_destroy_folder_t(folders);
     }
+    LIBMTP_Release_Device(device);
   }
 
-  LIBMTP_Release_Device_List(device);
+  free(rawdevices);
   printf("OK.\n");
 
   return 0;
