@@ -666,7 +666,7 @@ LIBMTP_error_number_t LIBMTP_Detect_Raw_Devices(LIBMTP_raw_device_t ** devices,
 	retdevs[i].device_entry.device_flags = mtp_device_table[j].device_flags;
 
 	// This device is known to the developers
-	LIBMTP_ERROR("Device %d (VID=%04x and PID=%04x) is a %s %s.\n",
+	LIBMTP_INFO("Device %d (VID=%04x and PID=%04x) is a %s %s.\n",
 		i,
 		dev->libusb_device->descriptor.idVendor,
 		dev->libusb_device->descriptor.idProduct,
